@@ -162,6 +162,10 @@ extern "C" {
 #define min_key_size ulMinKeySize
 #define max_key_size ulMaxKeySize
 
+#define ck_rsa_pkcs_oaep_params _CK_RSA_PCKS_OAEP_PARAMS
+#define source_data pSourceData
+#define source_data_len ulSourceDataLen
+
 #define ck_rv_t CK_RV
 #define ck_notify_t CK_NOTIFY
 
@@ -673,6 +677,11 @@ typedef unsigned long ck_mechanism_type_t;
 #define CKM_X9_42_DH_PARAMETER_GEN	(0x2002)
 #define CKM_VENDOR_DEFINED		((unsigned long) (1 << 31))
 
+#define CKG_MGF1_SHA1    (0x00000001)
+#define CKG_MGF1_SHA256  (0x00000002)
+#define CKG_MGF1_SHA384  (0x00000003)
+#define CKG_MGF1_SHA512  (0x00000004)
+
 
 struct ck_mechanism
 {
@@ -689,6 +698,15 @@ struct ck_mechanism_info
   ck_flags_t flags;
 };
 
+
+struct ck_rsa_pkcs_oaep_params {
+  unsigned long hashAlg;
+  unsigned long mgf;
+  unsigned long src;
+  void *source_data;
+  unsigned long source_data_len;
+} ;
+
 #define CKF_HW			(1 << 0)
 #define CKF_ENCRYPT		(1 << 8)
 #define CKF_DECRYPT		(1 << 9)
@@ -1252,6 +1270,9 @@ typedef struct ck_mechanism *CK_MECHANISM_PTR;
 typedef struct ck_mechanism_info CK_MECHANISM_INFO;
 typedef struct ck_mechanism_info *CK_MECHANISM_INFO_PTR;
 
+typedef struct ck_rsa_pkcs_oaep_params CK_RSA_PKCS_OAEP_PARAMS;
+typedef struct ck_rsa_pkcs_oaep_params *CK_RSA_PKCS_OAEP_PARAMS_PTR;
+
 typedef struct ck_function_list CK_FUNCTION_LIST;
 typedef struct ck_function_list *CK_FUNCTION_LIST_PTR;
 typedef struct ck_function_list **CK_FUNCTION_LIST_PTR_PTR;
@@ -1324,6 +1345,8 @@ typedef struct ck_c_initialize_args *CK_C_INITIALIZE_ARGS_PTR;
 #undef min_key_size
 #undef max_key_size
 
+#undef ck_rsa_pkcs_oaep_params
+
 #undef ck_rv_t
 #undef ck_notify_t
 
